package Business::IBAN;

require 5.005_62;
use strict;
use vars qw(%iso3166 %iso_rev @ISA @EXPORT_OK @EXPORT $VERSION @errors);

require Exporter;

@ISA = qw(Exporter);

@EXPORT_OK = ( );

@EXPORT = qw();
$VERSION = '0.01';
use constant CTR => 0;
use constant BBAN => 1;
use constant AC => 2;
use constant ISO => 3;
use constant LENGTH => 4;

sub new {
  my $type = shift;
  my $self  = {};
  bless($self, $type);
  return $self;

}
use Data::Dumper;
# --------------------------------------
sub getIBAN {
	my $self = shift;
	my $args = shift;
	my $ctr = $args->{COUNTRY};
	my $iso = $args->{ISO};
	my $bban = $args->{BBAN};
	my $ac = $args->{AC};
	delete $self->{ERRORS};
	delete $self->{COUNTRY};
	push @{$self->{ERRORS}}, CTR unless ($iso || $ctr);
	push @{$self->{ERRORS}}, BBAN unless $bban;
	push @{$self->{ERRORS}}, AC unless $ac;
#print Dumper $self;
	return if $self->{ERRORS};
	$iso =~ tr/ //d;
	$bban =~ tr/ //d;
	$ac =~ tr/ //d;

	$ctr = uc $ctr;
	unless ($iso) {
		$iso = $iso3166{$ctr};
	}
	unless ($iso) {
		my $key = (grep {m/\Q$ctr/i} keys %iso3166)[0];
		push @{$self->{ERRORS}}, ISO unless $key;
		return if $self->{ERRORS};
		$iso = $iso3166{$key};
		$self->{COUNTRY} = $key;
	}
	$self->{COUNTRY} ||= $iso_rev{$iso};
	my $cv = uc $iso;
	$cv =~ s/([A-Z])/(ord $1)-55/eg;
	my $no = sprintf "%08s%010s%4s00", $bban,$ac, $cv;
	my $mod = 98 - ($no % 97);
	substr($no,-6,6) = "";
	$no = 'IBAN '.$iso.$mod.$no;
	
	return $no;
}
# --------------------------------------
sub getError {
	my $self = shift;
	return unless $self->{ERRORS};
	return @{$self->{ERRORS}};
}
# --------------------------------------
sub printError {
	my $self = shift;
	return unless $self->{ERRORS};
	print "$errors[$_]\n" for @{$self->{ERRORS}};
}
# --------------------------------------
sub country {
	my $self = shift;
	return $self->{COUNTRY};
}
# --------------------------------------
sub valid {
	my $self = shift;
	my $ib = shift;
	delete $self->{ERRORS};
	$ib =~ tr/ //d;
	$ib =~ s/^IBAN//i;
	push @{$self->{ERRORS}}, LENGTH unless length($ib) == 22;
	return if $self->{ERRORS};
	my $iso = substr($ib,0,2,"");
	$iso =~ s/([A-Z])/(ord $1)-55/eg;
	my $check = substr($ib,0,2,"");
	$ib .= "$iso$check";
	return $ib % 97 ? 1 : 0;
}
# --------------------------------------

@errors = (
	"No Country or Iso-Code",
	"No BBAN (Bank-Number)",
	"No AC (Account Number)",
	"Could not find country",
	"Length if IBAN must be 22",
);
%iso3166 = (
	"AFGHANISTAN"                                    => "AF",
	"ALBANIA"                                        => "AL",
	"ALGERIA"                                        => "DZ",
	"AMERICAN SAMOA"                                 => "AS",
	"ANDORRA"                                        => "AD",
	"ANGOLA"                                         => "AO",
	"ANGUILLA"                                       => "AI",
	"ANTARCTICA"                                     => "AQ",
	"ANTIGUA AND BARBUDA"                            => "AG",
	"ARGENTINA"                                      => "AR",
	"ARMENIA"                                        => "AM",
	"ARUBA"                                          => "AW",
	"AUSTRALIA"                                      => "AU",
	"AUSTRIA"                                        => "AT",
	"AZERBAIJAN"                                     => "AZ",
	"BAHAMAS"                                        => "BS",
	"BAHRAIN"                                        => "BH",
	"BANGLADESH"                                     => "BD",
	"BARBADOS"                                       => "BB",
	"BELARUS"                                        => "BY",
	"BELGIUM"                                        => "BE",
	"BELIZE"                                         => "BZ",
	"BENIN"                                          => "BJ",
	"BERMUDA"                                        => "BM",
	"BHUTAN"                                         => "BT",
	"BOLIVIA"                                        => "BO",
	"BOSNIA AND HERZEGOVINA"                         => "BA",
	"BOTSWANA"                                       => "BW",
	"BOUVET ISLAND"                                  => "BV",
	"BRAZIL"                                         => "BR",
	"BRITISH INDIAN OCEAN TERRITORY"                 => "IO",
	"BRUNEI DARUSSALAM"                              => "BN",
	"BULGARIA"                                       => "BG",
	"BURKINA FASO"                                   => "BF",
	"BURUNDI"                                        => "BI",
	"CAMBODIA"                                       => "KH",
	"CAMEROON"                                       => "CM",
	"CANADA"                                         => "CA",
	"CAPE VERDE"                                     => "CV",
	"CAYMAN ISLANDS"                                 => "KY",
	"CENTRAL AFRICAN REPUBLIC"                       => "CF",
	"CHAD"                                           => "TD",
	"CHILE"                                          => "CL",
	"CHINA"                                          => "CN",
	"CHRISTMAS ISLAND"                               => "CX",
	"COCOS (KEELING) ISLANDS"                        => "CC",
	"COLOMBIA"                                       => "CO",
	"COMOROS"                                        => "KM",
	"CONGO"                                          => "CG",
	"CONGO, THE DEMOCRATIC REPUBLIC OF THE"          => "CD",
	"COOK ISLANDS"                                   => "CK",
	"COSTA RICA"                                     => "CR",
	"CTE D'IVOIRE"                                  => "CI",
	"CROATIA"                                        => "HR",
	"CUBA"                                           => "CU",
	"CYPRUS"                                         => "CY",
	"CZECH REPUBLIC"                                 => "CZ",
	"DENMARK"                                        => "DK",
	"DJIBOUTI"                                       => "DJ",
	"DOMINICA"                                       => "DM",
	"DOMINICAN REPUBLIC"                             => "DO",
	"EAST TIMOR"                                     => "TP",
	"ECUADOR"                                        => "EC",
	"EGYPT"                                          => "EG",
	"EL SALVADOR"                                    => "SV",
	"EQUATORIAL GUINEA"                              => "GQ",
	"ERITREA"                                        => "ER",
	"ESTONIA"                                        => "EE",
	"ETHIOPIA"                                       => "ET",
	"FALKLAND ISLANDS (MALVINAS)"                    => "FK",
	"FAROE ISLANDS"                                  => "FO",
	"FIJI"                                           => "FJ",
	"FINLAND"                                        => "FI",
	"FRANCE"                                         => "FR",
	"FRENCH GUIANA"                                  => "GF",
	"FRENCH POLYNESIA"                               => "PF",
	"FRENCH SOUTHERN TERRITORIES"                    => "TF",
	"GABON"                                          => "GA",
	"GAMBIA"                                         => "GM",
	"GEORGIA"                                        => "GE",
	"GERMANY"                                        => "DE",
	"GHANA"                                          => "GH",
	"GIBRALTAR"                                      => "GI",
	"GREECE"                                         => "GR",
	"GREENLAND"                                      => "GL",
	"GRENADA"                                        => "GD",
	"GUADELOUPE"                                     => "GP",
	"GUAM"                                           => "GU",
	"GUATEMALA"                                      => "GT",
	"GUINEA"                                         => "GN",
	"GUINEA-BISSAU"                                  => "GW",
	"GUYANA"                                         => "GY",
	"HAITI"                                          => "HT",
	"HEARD ISLAND AND MCDONALD ISLANDS"              => "HM",
	"HOLY SEE (VATICAN CITY STATE)"                  => "VA",
	"HONDURAS"                                       => "HN",
	"HONG KONG"                                      => "HK",
	"HUNGARY"                                        => "HU",
	"ICELAND"                                        => "IS",
	"INDIA"                                          => "IN",
	"INDONESIA"                                      => "ID",
	"IRAN, ISLAMIC REPUBLIC OF"                      => "IR",
	"IRAQ"                                           => "IQ",
	"IRELAND"                                        => "IE",
	"ISRAEL"                                         => "IL",
	"ITALY"                                          => "IT",
	"JAMAICA"                                        => "JM",
	"JAPAN"                                          => "JP",
	"JORDAN"                                         => "JO",
	"KAZAKSTAN"                                      => "KZ",
	"KENYA"                                          => "KE",
	"KIRIBATI"                                       => "KI",
	"KOREA, DEMOCRATIC PEOPLE'S REPUBLIC OF"         => "KP",
	"KOREA, REPUBLIC OF"                             => "KR",
	"KUWAIT"                                         => "KW",
	"KYRGYZSTAN"                                     => "KG",
	"LAO PEOPLE'S DEMOCRATIC REPUBLIC"               => "LA",
	"LATVIA"                                         => "LV",
	"LEBANON"                                        => "LB",
	"LESOTHO"                                        => "LS",
	"LIBERIA"                                        => "LR",
	"LIBYAN ARAB JAMAHIRIYA"                         => "LY",
	"LIECHTENSTEIN"                                  => "LI",
	"LITHUANIA"                                      => "LT",
	"LUXEMBOURG"                                     => "LU",
	"MACAU"                                          => "MO",
	"MACEDONIA, THE FORMER YUGOSLAV REPUBLIC OF"     => "MK",
	"MADAGASCAR"                                     => "MG",
	"MALAWI"                                         => "MW",
	"MALAYSIA"                                       => "MY",
	"MALDIVES"                                       => "MV",
	"MALI"                                           => "ML",
	"MALTA"                                          => "MT",
	"MARSHALL ISLANDS"                               => "MH",
	"MARTINIQUE"                                     => "MQ",
	"MAURITANIA"                                     => "MR",
	"MAURITIUS"                                      => "MU",
	"MAYOTTE"                                        => "YT",
	"MEXICO"                                         => "MX",
	"MICRONESIA, FEDERATED STATES OF"                => "FM",
	"MOLDOVA, REPUBLIC OF"                           => "MD",
	"MONACO"                                         => "MC",
	"MONGOLIA"                                       => "MN",
	"MONTSERRAT"                                     => "MS",
	"MOROCCO"                                        => "MA",
	"MOZAMBIQUE"                                     => "MZ",
	"MYANMAR"                                        => "MM",
	"NAMIBIA"                                        => "NA",
	"NAURU"                                          => "NR",
	"NEPAL"                                          => "NP",
	"NETHERLANDS"                                    => "NL",
	"NETHERLANDS ANTILLES"                           => "AN",
	"NEW CALEDONIA"                                  => "NC",
	"NEW ZEALAND"                                    => "NZ",
	"NICARAGUA"                                      => "NI",
	"NIGER"                                          => "NE",
	"NIGERIA"                                        => "NG",
	"NIUE"                                           => "NU",
	"NORFOLK ISLAND"                                 => "NF",
	"NORTHERN MARIANA ISLANDS"                       => "MP",
	"NORWAY"                                         => "NO",
	"OMAN"                                           => "OM",
	"PAKISTAN"                                       => "PK",
	"PALAU"                                          => "PW",
	"PALESTINIAN TERRITORY, OCCUPIED"                => "PS",
	"PANAMA"                                         => "PA",
	"PAPUA NEW GUINEA"                               => "PG",
	"PARAGUAY"                                       => "PY",
	"PERU"                                           => "PE",
	"PHILIPPINES"                                    => "PH",
	"PITCAIRN"                                       => "PN",
	"POLAND"                                         => "PL",
	"PORTUGAL"                                       => "PT",
	"PUERTO RICO"                                    => "PR",
	"QATAR"                                          => "QA",
	"RUNION"                                        => "RE",
	"ROMANIA"                                        => "RO",
	"RUSSIAN FEDERATION"                             => "RU",
	"RWANDA"                                         => "RW",
	"SAINT HELENA"                                   => "SH",
	"SAINT KITTS AND NEVIS"                          => "KN",
	"SAINT LUCIA"                                    => "LC",
	"SAINT PIERRE AND MIQUELON"                      => "PM",
	"SAINT VINCENT AND THE GRENADINES"               => "VC",
	"SAMOA"                                          => "WS",
	"SAN MARINO"                                     => "SM",
	"SAO TOME AND PRINCIPE"                          => "ST",
	"SAUDI ARABIA"                                   => "SA",
	"SENEGAL"                                        => "SN",
	"SEYCHELLES"                                     => "SC",
	"SIERRA LEONE"                                   => "SL",
	"SINGAPORE"                                      => "SG",
	"SLOVAKIA"                                       => "SK",
	"SLOVENIA"                                       => "SI",
	"SOLOMON ISLANDS"                                => "SB",
	"SOMALIA"                                        => "SO",
	"SOUTH AFRICA"                                   => "ZA",
	"SOUTH GEORGIA AND THE SOUTH SANDWICH ISLANDS"   => "GS",
	"SPAIN"                                          => "ES",
	"SRI LANKA"                                      => "LK",
	"SUDAN"                                          => "SD",
	"SURINAME"                                       => "SR",
	"SVALBARD AND JAN MAYEN"                         => "SJ",
	"SWAZILAND"                                      => "SZ",
	"SWEDEN"                                         => "SE",
	"SWITZERLAND"                                    => "CH",
	"SYRIAN ARAB REPUBLIC"                           => "SY",
	"TAIWAN, PROVINCE OF CHINA"                      => "TW",
	"TAJIKISTAN"                                     => "TJ",
	"TANZANIA, UNITED REPUBLIC OF"                   => "TZ",
	"THAILAND"                                       => "TH",
	"TOGO"                                           => "TG",
	"TOKELAU"                                        => "TK",
	"TONGA"                                          => "TO",
	"TRINIDAD AND TOBAGO"                            => "TT",
	"TUNISIA"                                        => "TN",
	"TURKEY"                                         => "TR",
	"TURKMENISTAN"                                   => "TM",
	"TURKS AND CAICOS ISLANDS"                       => "TC",
	"TUVALU"                                         => "TV",
	"UGANDA"                                         => "UG",
	"UKRAINE"                                        => "UA",
	"UNITED ARAB EMIRATES"                           => "AE",
	"UNITED KINGDOM"                                 => "GB",
	"UNITED STATES"                                  => "US",
	"UNITED STATES MINOR OUTLYING ISLANDS"           => "UM",
	"URUGUAY"                                        => "UY",
	"UZBEKISTAN"                                     => "UZ",
	"VANUATU"                                        => "VU",
	"VATICAN CITY STATE"                             => "VA",
	"VENEZUELA"                                      => "VE",
	"VIET NAM"                                       => "VN",
	"VIRGIN ISLANDS, BRITISH"                        => "VG",
	"VIRGIN ISLANDS, U.S."                           => "VI",
	"WALLIS AND FUTUNA"                              => "WF",
	"WESTERN SAHARA"                                 => "EH",
	"YEMEN"                                          => "YE",
	"YUGOSLAVIA"                                     => "YU",
	"ZAIRE"                                          => "CD",
	"ZAMBIA"                                         => "ZM",
	"ZIMBABWE"                                       => "ZW",
);
%iso_rev = reverse %iso3166;
$iso_rev{CD} = "CONGO, THE DEMOCRATIC REPUBLIC OF THE";
$iso_rev{VA} = "HOLY SEE";

1;
__END__

=head1 NAME

Business::IBAN - Validate and generate IBANs

=head1 SYNOPSIS

  use Business::IBAN;
  my $iban = Business::IBAN->new();
  my $ib = $iban->getIBAN(
  {
    ISO => "DE",  # or COUNTRY => "GERMANY",
    BBAN => 12345678,
    AC => "1234567890",
  });
  if ($ib) {
    print "IBAN is $ib\n";
  }
  else {
    $iban->printError();
    # or
    my @errors = $iban->getError();
    # print your own error messages (for description of error-
    # codes see section ERROR-CODES
  }

=head1 DESCRIPTION

With this module you can validate IBANs (International Bank
Account Number) like "IBAN DE12345678901234567890".
Note that this dos not (and cannot) assure that the bank
account exists or that the bank account number for the
bank itself is valid.
You can also create an IBAN if you supply your bank number,
bank account number and either your country code (ISO3166)
or the english name for your country.

=head2 EXPORT

None by default. All methods are accessed over the object.


=head2 ERROR-CODES

You can print your own error-messages. The array you get from
  my @errors = $iban->getError();
are numbers which stand for the following errors:
  1: No Country or Iso-Code given
  2: No BBAN (Bank-Number)
  3: No AC (Account Number)
  4: Could not find country
  5: Length if IBAN must be 22

=head1 AUTHOR

Tina Mueller. tinita@cpan.org

=head1 SEE ALSO

perl(1).

=cut
